@@ -48,7 +48,7 @@ namespace Do.Platform.Linux
 {
 	internal static class AssemblyInfo
 	{
-		public const string InstallPrefix = "/home/raof/.local";
-		public const string InstallData = "/home/raof/.local/share";
+		public const string InstallPrefix = "/usr/local";
+		public const string InstallData = "/usr/local/share";
 	}
 }
