@@ -19,6 +19,7 @@
 
 using System;
 using System.Linq;
+using System.Reflection;
 using System.Collections.Generic;
 
 using Mono.Addins;
@@ -32,18 +33,20 @@ namespace Do.Platform
 
 	public class Services
 	{
-
 		static ICoreService core;
 		static PathsService paths;
+		static INetworkService network;
 		static IWindowingService windowing;
 		static AbstractSystemService system;
+		static IKeyBindingService keybinder;
 		static IEnumerable<ILogService> logs;
 		static PreferencesFactory preferences;
-		static AbstractApplicationService application;
 		static IEnvironmentService environment;
 		static INotificationsService notifications;
+		static IPluginManagerService plugin_manager;
+		static AbstractApplicationService application;
 		static IUniverseFactoryService universe_factory;
-		static INetworkService network;
+		static AbstractPackageManagerService package_manager;
 
 		/// <summary>
 		/// Initializes the class. Must be called after Mono.Addins is initialized; if this is
@@ -59,14 +62,16 @@ namespace Do.Platform
 				// TODO find a better exception to throw.
 				throw new Exception ("AddinManager was initialized before Services.");
 			}
+			
 			AddinManager.AddExtensionNodeHandler ("/Do/Service", OnServiceChanged);
+			InitializeStrictServices ();
 		}
-
+		
 		/// <summary>
 		/// When a service is changed, we "dirty the cache".
 		/// </summary>
 		static void OnServiceChanged (object sender, ExtensionNodeEventArgs e)
-		{
+		{			
 			IService service = e.ExtensionObject as IService;
 
 			switch (e.Change) {
@@ -79,29 +84,29 @@ namespace Do.Platform
 			}
 
 			// Dirty the appropriate cache.
+			if (service is ILogService)
+				logs = null;
 			if (service is ICoreService)
 				core = null;
-			if (service is IEnvironmentService)
-				environment = null;
+			if (service is PathsService)
+				paths = null;
+			if (service is INetworkService)
+				network = null;
+			if (service is IWindowingService)
+				windowing = null;
 			// Although it is not obvious, this also takes care of the ISecurePreferences service.
 			if (service is IPreferencesService)
 				preferences = null;
-			if (service is ILogService)
-				logs = null;
-			if (service is IUniverseFactoryService)
-				universe_factory = null;
-			if (service is INotificationsService)
-				notifications = null;
-			if (service is IWindowingService)
-				windowing = null;
-			if (service is PathsService)
-				paths = null;
-			if (service is AbstractApplicationService)
-				application = null;
+			if (service is IEnvironmentService)
+				environment = null;
 			if (service is AbstractSystemService)
 				system = null;
-			if (service is INetworkService)
-				network = null;
+			if (service is INotificationsService)
+				notifications = null;
+			if (service is IUniverseFactoryService)
+				universe_factory = null;			
+			if (service is AbstractApplicationService)
+				application = null;
 		}
 
 		/// <summary>
@@ -185,8 +190,32 @@ namespace Do.Platform
 					network = LocateService<INetworkService, Default.NetworkService> ();
 				return network;
 			}
-		}	
-			
+		}
+		
+		public static AbstractPackageManagerService PackageManager {
+			get {
+				if (package_manager == null)
+					package_manager = LocateService<AbstractPackageManagerService, Default.DefaultPackageManagerService> ();
+				return package_manager;
+			}
+		}
+		
+		public static IPluginManagerService PluginManager {
+			get {
+				if (plugin_manager == null)
+					plugin_manager = LocateService<IPluginManagerService, Default.PluginManagerService> ();
+				return plugin_manager;
+			}
+		}
+		
+		public static IKeyBindingService Keybinder {
+			get {
+				if (keybinder == null)
+					keybinder = LocateService<IKeyBindingService, Default.KeyBindingService> ();
+				return keybinder;
+			}
+		}
+
 		public static PreferencesFactory Preferences {
 			get {
 				if (preferences == null) {
@@ -229,5 +258,20 @@ namespace Do.Platform
 				return Enumerable.Empty<TService> ();
 			}
 		}
+		
+		/// <summary>
+		/// loops through the Property members of this class, and if it's a strict service gets it's value.
+		/// This will in turn cause a LocateService call, and the appropriate service will be loaded.
+		/// </summary>
+		static void InitializeStrictServices ()
+		{
+			foreach (PropertyInfo property in typeof (Services).GetProperties ()) {
+				Type returnType = property.PropertyType;
+				if (returnType.GetInterface ("Do.Platform.IStrictService") != null) {
+					// this looks stupid, but this is how you call the method on static classes.
+					property.GetValue (null, null);
+				}
+			}
+		}
 	}
 }
