@@ -30,6 +30,7 @@ using Do;
 using Do.UI;
 using Do.Universe;
 using Do.Platform;
+using Do.Platform.Common;
 using Do.Interface;
 
 namespace Do.Core
@@ -103,101 +104,76 @@ namespace Do.Core
 
 		public void Initialize ()
 		{
+			if (Do.Preferences.Theme == "Docky") {
+				string message = Catalog.GetString (
+					"<b>Docky is no longer a Do theme!</b>\n" + 
+					"It is now available as a stand-alone application. " +
+					"Your GNOME Do theme has been reset to Classic. " +
+					"Please feel free to change it in Preferences."
+				);
+
+				Gtk.MessageDialog md = new Gtk.MessageDialog (null, Gtk.DialogFlags.DestroyWithParent, 
+					Gtk.MessageType.Info, Gtk.ButtonsType.Ok, true, message);
+				md.Response += (o, args) => md.Destroy ();
+				md.ShowAll ();
+				Do.Preferences.Theme = "Classic";
+			}
+			
 			SetTheme (Do.Preferences.Theme);
 			Do.Preferences.ThemeChanged += OnThemeChanged;
 			Screen.Default.CompositedChanged += OnCompositingChanged;
 			
 			// Register Shortcuts
-			// TODO: Localize the text here.
-			// Previous shortcuts
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("SummonKey", 
-						Catalog.GetString ("Summon GNOME Do"), 
-						OnSummonKeyPressEvent),
-					"<Super>space");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("TextModeKey",
-						Catalog.GetString ("Enter text mode"),
-						OnTextModePressEvent),
-					"period");
-			// New shortcuts
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("CopyKey",
-						Catalog.GetString ("Copy Text"),
-						OnCopyEvent),
-					"<Control>c");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("PasteKey",
-						Catalog.GetString ("Paste Text"),
-						OnPasteEvent),
-					"<Control>v");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("AlternateEscapeKey",
-						Catalog.GetString ("Alternate Escape"),
-						OnEscapeKeyPressEvent));
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("AlternateActivateKey",
-						Catalog.GetString ("Alternate Activate"),
-						OnActivateKeyPressEvent));
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("AlternateDeleteKey",
-						Catalog.GetString ("Alternate Delete"),
-						OnDeleteKeyPressEvent));
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("ShiftTabKey",
-						Catalog.GetString ("Previous Pane"),
-						OnShiftTabKeyPressEvent),
-					"ISO_Left_Tab");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("TabKey",
-						Catalog.GetString ("Next Pane"),
-						OnTabKeyPressEvent),
-					"Tab");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("UpKey",
-						Catalog.GetString ("Previous Result"),
-						OnUpKeyPressEvent),
-					"Up");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("DownKey",
-						Catalog.GetString ("Next Result"),
-						OnDownKeyPressEvent),
-					"Down");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("HomeKey",
-						Catalog.GetString ("First Result"),
-						OnHomeKeyPressEvent),
-					"Home");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("EndKey",
-						Catalog.GetString ("Last Result"),
-						OnEndKeyPressEvent),
-					"End");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("PageUpKey",
-						Catalog.GetString ("Previous 5 Results"),
-						OnPageUpKeyPressEvent),
-					"Page_Up");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("PageDownKey",
-						Catalog.GetString ("Next 5 Results"),
-						OnPageDownKeyPressEvent),
-					"Page_Down");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("LeftKey",
-						Catalog.GetString ("Step out of Item"),
-						OnLeftKeyPressEvent),
-					"Left");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("RightKey",
-						Catalog.GetString ("Browse Into Item"),
-						OnRightKeyPressEvent),
-					"Right");
-			Do.Keybindings.RegisterShortcut (
-					new Shortcut ("CommaKey",
-						Catalog.GetString ("Multiple selection"),
-						OnSelectionKeyPressEvent),
-					"comma");
+			SetupKeybindings ();
+		}
+		
+		void SetupKeybindings ()
+		{
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Summon Do"), "<Super>space", 
+				OnSummonKeyPressEvent, true));
+			
+			// this keybinding is disabled by default - note the empty keybinding
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Summon in Text Mode"), "",
+				OnTextModeSummonKeyPressEvent, true));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Enter Text Mode"), "period",
+				OnTextModePressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Clear"), "Escape",
+				OnClearKeyPressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Copy to Clipboard"), "<Control>c",
+				OnCopyEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Paste from Clipboard"), "<Control>v",
+				OnPasteEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Previous Pane"), "<Shift>Tab",
+				OnPreviousPanePressEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Next Pane"), "Tab",
+				OnNextPanePressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Previous Item"), "Up",
+				OnPreviousItemPressEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Next Item"), "Down",
+				OnNextItemPressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("First Item"), "Home",
+				OnFirstItemPressEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Last Item"), "End",
+				OnLastItemPressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Previous 5 Results"), "Page_Up",
+				OnNextItemPagePressEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Next 5 Results"), "Page_Down",
+				OnPreviousItemPagePressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Step Out of Item"), "Left",
+				OnStepOutItemPressEvent));
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Step Into Item"), "Right",
+				OnStepInItemPressEvent));
+			
+			Services.Keybinder.RegisterKeyBinding (new KeyBinding (Catalog.GetString ("Select Multiple Items"), "comma",
+				OnSelectionKeyPressEvent));
 		}
 
 		void OnSummoned ()
@@ -206,18 +182,20 @@ namespace Do.Core
 			Summoned (this, EventArgs.Empty);
 		}
 		
-		void OnCompositingChanged(object sender, EventArgs args)
+		void OnCompositingChanged (object sender, EventArgs args)
 		{
-			UnsetTheme();
-			SetTheme(Do.Preferences.Theme);
+			UnsetTheme ();
+			SetTheme (Do.Preferences.Theme);
 		}
 		
+		private string LastTheme { get; set; }
+		
 		void OnThemeChanged (object sender, PreferencesChangedEventArgs e)
 		{
-			string oldTheme = e.OldValue as string, newTheme = Do.Preferences.Theme;
+			string newTheme = e.Value as string;
 			
 			// Only change the theme of the old and new themes are different.
-			if (oldTheme != newTheme) {
+			if (LastTheme != newTheme && !string.IsNullOrEmpty (newTheme)) {
 				UnsetTheme ();
 				SetTheme (newTheme);
 			}
@@ -236,6 +214,7 @@ namespace Do.Core
 		void SetTheme (string themeName)
 		{
 			Log<Controller>.Debug ("Setting theme {0}", themeName);
+			LastTheme = themeName;
 			
 			Orientation = ControlOrientation.Vertical;
 
@@ -482,38 +461,22 @@ namespace Do.Core
 
 #region KeyPress Handling
 
-		Key IfVertical (Key ifKey, Key elseKey)
-		{
-			return Orientation == ControlOrientation.Vertical ? ifKey : elseKey;
-		}
-
-		Key UpKey    { get { return IfVertical (Key.Up, Key.Left); } }
-		Key DownKey  { get { return IfVertical (Key.Down, Key.Right); } }
-		Key LeftKey  { get { return IfVertical (Key.Left, Key.Up); } }
-		Key RightKey { get { return IfVertical (Key.Right, Key.Down); } }
-		
 		private void KeyPressWrap (EventKey evnt)
 		{
 			Key key = (Key) evnt.KeyValue;
-
 			
-			// Currently - only hardcoded are enter keys, escape and delete/backspace
-			if (key == Key.Escape) {
-				OnEscapeKeyPressEvent (evnt);
-			} else if (key == Key.Return ||
+			// Currently - only hardcoded are enter keys and delete/backspace
+			if (key == Key.Return ||
 				   key == Key.ISO_Enter ||
 				   key == Key.KP_Enter) {
 				OnActivateKeyPressEvent (evnt);
 			} else if (key == Key.Delete ||
 				   key == Key.BackSpace) {
 				OnDeleteKeyPressEvent (evnt);
-			} else {
+			} else if (Services.Keybinder.Bindings.Any (k => k.KeyString == Services.Keybinder.KeyEventToString (evnt))) {
 				// User set keybindings
-				Shortcut sc = Do.Keybindings.GetShortcutByKeycode (KeyEventToString (evnt));
-				if (sc != null) {
-					sc.Callback (evnt);
-					return;
-				}
+				Services.Keybinder.Bindings.First (k => k.KeyString == Services.Keybinder.KeyEventToString (evnt)).Callback (evnt);
+			} else {
 				OnInputKeyPressEvent (evnt);
 			}
 		}
@@ -569,11 +532,22 @@ namespace Do.Core
 		
 		void OnSummonKeyPressEvent (EventKey evnt)
 		{
-			Reset ();
-			Vanish ();
+			if (IsSummoned) {
+				Reset ();
+				Vanish ();
+			} else {
+				Summon ();
+			}
 		}
 		
-		void OnEscapeKeyPressEvent (EventKey evnt)
+		void OnTextModeSummonKeyPressEvent (EventKey evnt)
+		{
+			Summon (); 
+			SearchController.TextMode = true; 
+			UpdatePane (CurrentPane);
+		}
+		
+		void OnClearKeyPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (SearchController.TextType == TextModeType.Explicit) {
@@ -622,7 +596,7 @@ namespace Do.Core
 			}
 		}
 		
-		void OnLeftKeyPressEvent (EventKey evnt)
+		void OnStepOutItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (!SearchController.Results.Any ()) return;
@@ -637,7 +611,7 @@ namespace Do.Core
 		}
 
 		// Hmm.
-		void OnRightKeyPressEvent (EventKey evnt)
+		void OnStepInItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (!SearchController.Results.Any ()) return;
@@ -648,7 +622,7 @@ namespace Do.Core
 			if (SearchController.ItemChildSearch ()) GrowResults ();
 		}
 
-		void OnTabKeyPressEvent (EventKey evnt)
+		void OnNextPanePressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			ShrinkResults ();
@@ -661,7 +635,7 @@ namespace Do.Core
 			UpdatePane (CurrentPane);
 		}
 
-		void OnShiftTabKeyPressEvent (EventKey evnt)
+		void OnPreviousPanePressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			ShrinkResults ();
@@ -689,7 +663,7 @@ namespace Do.Core
 			UpdatePane (CurrentPane);
 		}
 		
-		void OnUpKeyPressEvent (EventKey evnt)
+		void OnPreviousItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (!results_grown) {
@@ -705,7 +679,7 @@ namespace Do.Core
 			}
 		}
 
-		void OnDownKeyPressEvent (EventKey evnt)
+		void OnNextItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			if (!results_grown) {
@@ -715,53 +689,29 @@ namespace Do.Core
 			SearchController.Cursor++;
 		}
 		
-		void OnHomeKeyPressEvent (EventKey evnt)
+		void OnFirstItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			SearchController.Cursor = 0;
 		}
 
-		void OnEndKeyPressEvent (EventKey evnt)
+		void OnLastItemPressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			SearchController.Cursor = SearchController.Results.Count - 1;
 		}
 
-		void OnPageUpKeyPressEvent (EventKey evnt)
+		void OnNextItemPagePressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			SearchController.Cursor -= 5;
 		}
 		
-		void OnPageDownKeyPressEvent (EventKey evnt)
+		void OnPreviousItemPagePressEvent (EventKey evnt)
 		{
 			im_context.Reset ();
 			SearchController.Cursor += 5;
 		}
-				
-		/// <summary>
-		/// Converts a keypress into a human readable string for comparing
-		/// against values in GConf.
-		/// </summary>
-		/// <param name="evnt">
-		/// A <see cref="EventKey"/>
-		/// </param>
-		/// <returns>
-		/// A <see cref="System.String"/> in the form "<Modifier>key"
-		/// </returns>
-		string KeyEventToString (EventKey evnt) {
-			string modifier = "";
-			if ((evnt.State & ModifierType.ControlMask) != 0) {
-				modifier += "<Control>";
-			}
-			if ((evnt.State & ModifierType.SuperMask) != 0) {
-				modifier += "<Super>";
-			}
-			if ((evnt.State & ModifierType.Mod1Mask) != 0) {
-				modifier += "<Alt>";
-			}
-			return modifier + evnt.Key.ToString ();
-		}
 #endregion
 		
 		/// <summary>
@@ -961,14 +911,21 @@ namespace Do.Core
 					item.IncreaseRelevance (itemQuery, null);
 
 				// Increase the relevance of the action /for each item/:
-				foreach (Item item in items)
-					action.IncreaseRelevance (actionQuery, item);
+				if (action != null) {
+					foreach (Item item in items)
+						action.IncreaseRelevance (actionQuery, item);
+				} else {
+					Log<Controller>.Debug ("Action is null in 2nd pane!");
+				}
 			} else {
 				// Act is in first pane.
 				// Increase the relevance of each item for the action.
 				foreach (Item item in items)
 					item.IncreaseRelevance (itemQuery, action);
-				action.IncreaseRelevance (actionQuery, null);
+				if (action != null)
+					action.IncreaseRelevance (actionQuery, null);
+				else
+					Log<Controller>.Debug ("Action is null in 1st pane!");
 			}
 
 			if (ThirdPaneVisible) {
@@ -1054,7 +1011,7 @@ namespace Do.Core
 			logo = "gnome-do.svg";
 
 			AboutDialog.Logo = IconProvider.PixbufFromIconName (logo, 140);
-			AboutDialog.Copyright = Catalog.GetString ("Copyright \xa9 2008 GNOME Do Developers");
+			AboutDialog.Copyright = Catalog.GetString ("Copyright \xa9 2009 GNOME Do Developers");
 			AboutDialog.Comments = Catalog.GetString ("Do things as quickly as possible\n" +
 				"(but no quicker) with your files, bookmarks,\n" +
 				"applications, music, contacts, and more!");
