@@ -18,71 +18,59 @@
  */
 
 using System;
+
 using Gtk;
 using Mono.Unix;
 
 using Do;
 using Do.Core;
+using Do.Platform;
+using Do.Universe;
+using Do.Interface;
 
 namespace Do.UI
 {
+
 	public class MainMenu : Gtk.Menu
 	{
+
 		static MainMenu instance;
 
 		public static MainMenu Instance {
 			get {
-				return instance ??
+				if (instance == null)
 					instance = new MainMenu ();
+				return instance;
 			}
 		}
 
 		int mainMenuX, mainMenuY;
 
-		public MainMenu ()
+		private MainMenu ()
 		{
-			MenuItem item;
-			// About menu item
-			item = new ImageMenuItem (Catalog.GetString ("_About Do"));
-			(item as ImageMenuItem).Image = new Image (Stock.About, IconSize.Menu);
-			Add (item);
-			item.CanFocus = false;
-			item.Activated += OnMainMenuAboutClicked;
-			
-			// Preferences menu item
-			item = new ImageMenuItem (Catalog.GetString ("_Preferences"));
-			(item as ImageMenuItem).Image = new Image (Stock.Preferences, IconSize.Menu);
-			Add (item);
-			item.CanFocus = false;
-			item.Activated += OnMainMenuPreferencesClicked;
-			
-			// Quit menu item
-			item = new ImageMenuItem (Catalog.GetString ("_Quit"));
-			(item as ImageMenuItem).Image = new Image (Stock.Quit, IconSize.Menu);
-			Add (item);
-			item.Activated += OnMainMenuQuitClicked;
-
+			foreach (IRunnableItem item in Services.Application.MainMenuItems)
+				Add (MenuItemFromRunnableItem (item));
 			ShowAll ();
 		}
 
-		protected void OnMainMenuQuitClicked (object o, EventArgs args)
+		MenuItem MenuItemFromRunnableItem (IRunnableItem item)
 		{
-			Do.Controller.Vanish ();
-			Application.Quit ();
-		}
-		
-		protected void OnMainMenuPreferencesClicked (object o, EventArgs args)
-		{
-			Do.Controller.ShowPreferences ();
-		}
-
-		protected void OnMainMenuAboutClicked (object o, EventArgs args)
-		{
-			Do.Controller.ShowAbout ();
+			int iconSize;
+			Gdk.Pixbuf icon;
+			ImageMenuItem menuItem;
+			
+			menuItem = new ImageMenuItem (item.Name);
+			Icon.SizeLookup (IconSize.Menu, out iconSize, out iconSize);
+			icon = IconProvider.PixbufFromIconName (item.Icon, iconSize);
+			menuItem.Image = new Image (icon);
+			menuItem.CanFocus = false;
+			menuItem.Activated += (sender, e) => item.Run ();
+			return menuItem;
 		}
 
 		public void PopupAtPosition (int x, int y)
 		{
+			menuPositioner = null;
 			mainMenuX = x;
 			mainMenuY = y;	
 			Popup (null, null, PositionMainMenu, 3, Gtk.Global.CurrentEventTime);
@@ -90,9 +78,22 @@ namespace Do.UI
 
 		private void PositionMainMenu (Menu menu, out int x, out int y, out bool push_in)
 		{
-			x = mainMenuX;
-			y = mainMenuY;
+			if (menuPositioner == null) {
+				x = mainMenuX;
+				y = mainMenuY;
+			} else {
+				Requisition menuReq = menu.SizeRequest ();
+				menuPositioner (menuReq.Height, menuReq.Width, out x, out y);
+			}
 			push_in = true;
 		}
+
+		PositionMenu menuPositioner;
+		
+		public void PopupWithPositioner (PositionMenu menuPositioner)
+		{
+			this.menuPositioner = menuPositioner;
+			Popup (null, null, PositionMainMenu, 3, Gtk.Global.CurrentEventTime);
+		}
 	}
-}
\ No newline at end of file
+}
